"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _Type:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _TypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Type.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNKNOWN: _Type.ValueType  # 0
    DIAL: _Type.ValueType  # 1
    DIAL_RESPONSE: _Type.ValueType  # 2

class Type(_Type, metaclass=_TypeEnumTypeWrapper):
    """Message types"""

UNKNOWN: Type.ValueType  # 0
DIAL: Type.ValueType  # 1
DIAL_RESPONSE: Type.ValueType  # 2
global___Type = Type

class _Status:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _StatusEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Status.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    OK: _Status.ValueType  # 0
    E_DIAL_ERROR: _Status.ValueType  # 1
    E_DIAL_REFUSED: _Status.ValueType  # 2
    E_DIAL_FAILED: _Status.ValueType  # 3
    E_INTERNAL_ERROR: _Status.ValueType  # 100

class Status(_Status, metaclass=_StatusEnumTypeWrapper):
    """Status codes"""

OK: Status.ValueType  # 0
E_DIAL_ERROR: Status.ValueType  # 1
E_DIAL_REFUSED: Status.ValueType  # 2
E_DIAL_FAILED: Status.ValueType  # 3
E_INTERNAL_ERROR: Status.ValueType  # 100
global___Status = Status

@typing.final
class Message(google.protobuf.message.Message):
    """Main message"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TYPE_FIELD_NUMBER: builtins.int
    DIAL_FIELD_NUMBER: builtins.int
    DIAL_RESPONSE_FIELD_NUMBER: builtins.int
    type: global___Type.ValueType
    @property
    def dial(self) -> global___DialRequest: ...
    @property
    def dial_response(self) -> global___DialResponse: ...
    def __init__(
        self,
        *,
        type: global___Type.ValueType = ...,
        dial: global___DialRequest | None = ...,
        dial_response: global___DialResponse | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["dial", b"dial", "dial_response", b"dial_response"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["dial", b"dial", "dial_response", b"dial_response", "type", b"type"]) -> None: ...

global___Message = Message

@typing.final
class DialRequest(google.protobuf.message.Message):
    """Dial request"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PEERS_FIELD_NUMBER: builtins.int
    @property
    def peers(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___PeerInfo]: ...
    def __init__(
        self,
        *,
        peers: collections.abc.Iterable[global___PeerInfo] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["peers", b"peers"]) -> None: ...

global___DialRequest = DialRequest

@typing.final
class DialResponse(google.protobuf.message.Message):
    """Dial response"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STATUS_FIELD_NUMBER: builtins.int
    PEERS_FIELD_NUMBER: builtins.int
    status: global___Status.ValueType
    @property
    def peers(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___PeerInfo]: ...
    def __init__(
        self,
        *,
        status: global___Status.ValueType = ...,
        peers: collections.abc.Iterable[global___PeerInfo] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["peers", b"peers", "status", b"status"]) -> None: ...

global___DialResponse = DialResponse

@typing.final
class PeerInfo(google.protobuf.message.Message):
    """Peer information"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    ADDRS_FIELD_NUMBER: builtins.int
    SUCCESS_FIELD_NUMBER: builtins.int
    id: builtins.bytes
    success: builtins.bool
    @property
    def addrs(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.bytes]: ...
    def __init__(
        self,
        *,
        id: builtins.bytes = ...,
        addrs: collections.abc.Iterable[builtins.bytes] | None = ...,
        success: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["addrs", b"addrs", "id", b"id", "success", b"success"]) -> None: ...

global___PeerInfo = PeerInfo
