# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: libp2p/discovery/rendezvous/pb/rendezvous.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n/libp2p/discovery/rendezvous/pb/rendezvous.proto\x12\rrendezvous.pb\"\xa8\x0c\n\x07Message\x12\x30\n\x04type\x18\x01 \x01(\x0e\x32\".rendezvous.pb.Message.MessageType\x12\x31\n\x08register\x18\x02 \x01(\x0b\x32\x1f.rendezvous.pb.Message.Register\x12\x41\n\x10registerResponse\x18\x03 \x01(\x0b\x32\'.rendezvous.pb.Message.RegisterResponse\x12\x35\n\nunregister\x18\x04 \x01(\x0b\x32!.rendezvous.pb.Message.Unregister\x12\x31\n\x08\x64iscover\x18\x05 \x01(\x0b\x32\x1f.rendezvous.pb.Message.Discover\x12\x41\n\x10\x64iscoverResponse\x18\x06 \x01(\x0b\x32\'.rendezvous.pb.Message.DiscoverResponse\x12\x43\n\x11\x64iscoverSubscribe\x18\x64 \x01(\x0b\x32(.rendezvous.pb.Message.DiscoverSubscribe\x12S\n\x19\x64iscoverSubscribeResponse\x18\x65 \x01(\x0b\x32\x30.rendezvous.pb.Message.DiscoverSubscribeResponse\x1a%\n\x08PeerInfo\x12\n\n\x02id\x18\x01 \x01(\x0c\x12\r\n\x05\x61\x64\x64rs\x18\x02 \x03(\x0c\x1aR\n\x08Register\x12\n\n\x02ns\x18\x01 \x01(\t\x12-\n\x04peer\x18\x02 \x01(\x0b\x32\x1f.rendezvous.pb.Message.PeerInfo\x12\x0b\n\x03ttl\x18\x03 \x01(\x03\x1aj\n\x10RegisterResponse\x12\x35\n\x06status\x18\x01 \x01(\x0e\x32%.rendezvous.pb.Message.ResponseStatus\x12\x12\n\nstatusText\x18\x02 \x01(\t\x12\x0b\n\x03ttl\x18\x03 \x01(\x03\x1a$\n\nUnregister\x12\n\n\x02ns\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\x0c\x1a\x35\n\x08\x44iscover\x12\n\n\x02ns\x18\x01 \x01(\t\x12\r\n\x05limit\x18\x02 \x01(\x03\x12\x0e\n\x06\x63ookie\x18\x03 \x01(\x0c\x1a\xa5\x01\n\x10\x44iscoverResponse\x12\x36\n\rregistrations\x18\x01 \x03(\x0b\x32\x1f.rendezvous.pb.Message.Register\x12\x0e\n\x06\x63ookie\x18\x02 \x01(\x0c\x12\x35\n\x06status\x18\x03 \x01(\x0e\x32%.rendezvous.pb.Message.ResponseStatus\x12\x12\n\nstatusText\x18\x04 \x01(\t\x1a\x45\n\x11\x44iscoverSubscribe\x12$\n\x1csupported_subscription_types\x18\x01 \x03(\t\x12\n\n\x02ns\x18\x02 \x01(\t\x1a\x9f\x01\n\x19\x44iscoverSubscribeResponse\x12\x19\n\x11subscription_type\x18\x01 \x01(\t\x12\x1c\n\x14subscription_details\x18\x02 \x01(\t\x12\x35\n\x06status\x18\x03 \x01(\x0e\x32%.rendezvous.pb.Message.ResponseStatus\x12\x12\n\nstatusText\x18\x04 \x01(\t\"\xa0\x01\n\x0bMessageType\x12\x0c\n\x08REGISTER\x10\x00\x12\x15\n\x11REGISTER_RESPONSE\x10\x01\x12\x0e\n\nUNREGISTER\x10\x02\x12\x0c\n\x08\x44ISCOVER\x10\x03\x12\x15\n\x11\x44ISCOVER_RESPONSE\x10\x04\x12\x16\n\x12\x44ISCOVER_SUBSCRIBE\x10\x64\x12\x1f\n\x1b\x44ISCOVER_SUBSCRIBE_RESPONSE\x10\x65\"\xb5\x01\n\x0eResponseStatus\x12\x06\n\x02OK\x10\x00\x12\x17\n\x13\x45_INVALID_NAMESPACE\x10\x64\x12\x17\n\x13\x45_INVALID_PEER_INFO\x10\x65\x12\x11\n\rE_INVALID_TTL\x10\x66\x12\x14\n\x10\x45_INVALID_COOKIE\x10g\x12\x15\n\x10\x45_NOT_AUTHORIZED\x10\xc8\x01\x12\x15\n\x10\x45_INTERNAL_ERROR\x10\xac\x02\x12\x12\n\rE_UNAVAILABLE\x10\x90\x03\"H\n\x12RegistrationRecord\x12\n\n\x02id\x18\x01 \x01(\t\x12\r\n\x05\x61\x64\x64rs\x18\x02 \x03(\x0c\x12\n\n\x02ns\x18\x03 \x01(\t\x12\x0b\n\x03ttl\x18\x04 \x01(\x03\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nanolib.discovery.rendezvous.pb.rendezvous_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_MESSAGE']._serialized_start=67
  _globals['_MESSAGE']._serialized_end=1643
  _globals['_MESSAGE_PEERINFO']._serialized_start=573
  _globals['_MESSAGE_PEERINFO']._serialized_end=610
  _globals['_MESSAGE_REGISTER']._serialized_start=612
  _globals['_MESSAGE_REGISTER']._serialized_end=694
  _globals['_MESSAGE_REGISTERRESPONSE']._serialized_start=696
  _globals['_MESSAGE_REGISTERRESPONSE']._serialized_end=802
  _globals['_MESSAGE_UNREGISTER']._serialized_start=804
  _globals['_MESSAGE_UNREGISTER']._serialized_end=840
  _globals['_MESSAGE_DISCOVER']._serialized_start=842
  _globals['_MESSAGE_DISCOVER']._serialized_end=895
  _globals['_MESSAGE_DISCOVERRESPONSE']._serialized_start=898
  _globals['_MESSAGE_DISCOVERRESPONSE']._serialized_end=1063
  _globals['_MESSAGE_DISCOVERSUBSCRIBE']._serialized_start=1065
  _globals['_MESSAGE_DISCOVERSUBSCRIBE']._serialized_end=1134
  _globals['_MESSAGE_DISCOVERSUBSCRIBERESPONSE']._serialized_start=1137
  _globals['_MESSAGE_DISCOVERSUBSCRIBERESPONSE']._serialized_end=1296
  _globals['_MESSAGE_MESSAGETYPE']._serialized_start=1299
  _globals['_MESSAGE_MESSAGETYPE']._serialized_end=1459
  _globals['_MESSAGE_RESPONSESTATUS']._serialized_start=1462
  _globals['_MESSAGE_RESPONSESTATUS']._serialized_end=1643
  _globals['_REGISTRATIONRECORD']._serialized_start=1645
  _globals['_REGISTRATIONRECORD']._serialized_end=1717
# @@protoc_insertion_point(module_scope)
