# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: libp2p/peer/persistent/pb/persistent_peerstore.proto
# Protobuf Python Version: 5.29.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    3,
    '',
    'libp2p/peer/persistent/pb/persistent_peerstore.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nanolib.peer.pb import crypto_pb2 as libp2p_dot_peer_dot_pb_dot_crypto__pb2
from nanolib.peer.pb import envelope_pb2 as libp2p_dot_peer_dot_pb_dot_envelope__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n4libp2p/peer/persistent/pb/persistent_peerstore.proto\x12\x19libp2p.peer.persistent.pb\x1a\x1blibp2p/peer/pb/crypto.proto\x1a\x1dlibp2p/peer/pb/envelope.proto\"\"\n\rPeerAddresses\x12\x11\n\taddresses\x18\x01 \x03(\x0c\"\"\n\rPeerProtocols\x12\x11\n\tprotocols\x18\x01 \x03(\t\"\x88\x01\n\x0cPeerMetadata\x12G\n\x08metadata\x18\x01 \x03(\x0b\x32\x35.libp2p.peer.persistent.pb.PeerMetadata.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x0c:\x02\x38\x01\"x\n\x08PeerKeys\x12\x34\n\npublic_key\x18\x01 \x01(\x0b\x32 .libp2p.peer.pb.crypto.PublicKey\x12\x36\n\x0bprivate_key\x18\x02 \x01(\x0b\x32!.libp2p.peer.pb.crypto.PrivateKey\"!\n\x0bPeerLatency\x12\x12\n\nlatency_ns\x18\x01 \x01(\x03\"A\n\x0cPeerEnvelope\x12\x31\n\x08\x65nvelope\x18\x01 \x01(\x0b\x32\x1f.libp2p.peer.pb.record.Envelope\"\x8d\x01\n\x0fPeerRecordState\x12?\n\x05state\x18\x01 \x01(\x0e\x32\x30.libp2p.peer.persistent.pb.PeerRecordState.State\"9\n\x05State\x12\x0b\n\x07UNKNOWN\x10\x00\x12\t\n\x05VALID\x10\x01\x12\x0b\n\x07INVALID\x10\x02\x12\x0b\n\x07\x45XPIRED\x10\x03\x42\x37Z5github.com/libp2p/py-libp2p/libp2p/peer/persistent/pbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nanolib.peer.persistent.pb.persistent_peerstore_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z5github.com/libp2p/py-libp2p/libp2p/peer/persistent/pb'
  _globals['_PEERMETADATA_METADATAENTRY']._loaded_options = None
  _globals['_PEERMETADATA_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_PEERADDRESSES']._serialized_start=143
  _globals['_PEERADDRESSES']._serialized_end=177
  _globals['_PEERPROTOCOLS']._serialized_start=179
  _globals['_PEERPROTOCOLS']._serialized_end=213
  _globals['_PEERMETADATA']._serialized_start=216
  _globals['_PEERMETADATA']._serialized_end=352
  _globals['_PEERMETADATA_METADATAENTRY']._serialized_start=305
  _globals['_PEERMETADATA_METADATAENTRY']._serialized_end=352
  _globals['_PEERKEYS']._serialized_start=354
  _globals['_PEERKEYS']._serialized_end=474
  _globals['_PEERLATENCY']._serialized_start=476
  _globals['_PEERLATENCY']._serialized_end=509
  _globals['_PEERENVELOPE']._serialized_start=511
  _globals['_PEERENVELOPE']._serialized_end=576
  _globals['_PEERRECORDSTATE']._serialized_start=579
  _globals['_PEERRECORDSTATE']._serialized_end=720
  _globals['_PEERRECORDSTATE_STATE']._serialized_start=663
  _globals['_PEERRECORDSTATE_STATE']._serialized_end=720
# @@protoc_insertion_point(module_scope)
