"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Propose(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RAND_FIELD_NUMBER: builtins.int
    PUBLIC_KEY_FIELD_NUMBER: builtins.int
    EXCHANGES_FIELD_NUMBER: builtins.int
    CIPHERS_FIELD_NUMBER: builtins.int
    HASHES_FIELD_NUMBER: builtins.int
    rand: builtins.bytes
    public_key: builtins.bytes
    exchanges: builtins.str
    ciphers: builtins.str
    hashes: builtins.str
    def __init__(
        self,
        *,
        rand: builtins.bytes | None = ...,
        public_key: builtins.bytes | None = ...,
        exchanges: builtins.str | None = ...,
        ciphers: builtins.str | None = ...,
        hashes: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["ciphers", b"ciphers", "exchanges", b"exchanges", "hashes", b"hashes", "public_key", b"public_key", "rand", b"rand"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["ciphers", b"ciphers", "exchanges", b"exchanges", "hashes", b"hashes", "public_key", b"public_key", "rand", b"rand"]) -> None: ...

global___Propose = Propose

@typing.final
class Exchange(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EPHEMERAL_PUBLIC_KEY_FIELD_NUMBER: builtins.int
    SIGNATURE_FIELD_NUMBER: builtins.int
    ephemeral_public_key: builtins.bytes
    signature: builtins.bytes
    def __init__(
        self,
        *,
        ephemeral_public_key: builtins.bytes | None = ...,
        signature: builtins.bytes | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["ephemeral_public_key", b"ephemeral_public_key", "signature", b"signature"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["ephemeral_public_key", b"ephemeral_public_key", "signature", b"signature"]) -> None: ...

global___Exchange = Exchange
