# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: libp2p/bitswap/pb/dag_pb.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1elibp2p/bitswap/pb/dag_pb.proto\x12\x06\x64\x61g_pb\"3\n\x06PBLink\x12\x0c\n\x04Hash\x18\x01 \x01(\x0c\x12\x0c\n\x04Name\x18\x02 \x01(\t\x12\r\n\x05Tsize\x18\x03 \x01(\x04\"5\n\x06PBNode\x12\x1d\n\x05Links\x18\x02 \x03(\x0b\x32\x0e.dag_pb.PBLink\x12\x0c\n\x04\x44\x61ta\x18\x01 \x01(\x0c')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nanolib.bitswap.pb.dag_pb_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_PBLINK']._serialized_start=42
  _globals['_PBLINK']._serialized_end=93
  _globals['_PBNODE']._serialized_start=95
  _globals['_PBNODE']._serialized_end=148
# @@protoc_insertion_point(module_scope)
