"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import libp2p.crypto.pb.crypto_pb2
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Exchange(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    PUBKEY_FIELD_NUMBER: builtins.int
    id: builtins.bytes
    @property
    def pubkey(self) -> libp2p.crypto.pb.crypto_pb2.PublicKey: ...
    def __init__(
        self,
        *,
        id: builtins.bytes | None = ...,
        pubkey: libp2p.crypto.pb.crypto_pb2.PublicKey | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["id", b"id", "pubkey", b"pubkey"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["id", b"id", "pubkey", b"pubkey"]) -> None: ...

global___Exchange = Exchange
