# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: libp2p/bitswap/pb/bitswap.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1flibp2p/bitswap/pb/bitswap.proto\x12\nbitswap.pb\"\xe8\x04\n\x07Message\x12.\n\x08wantlist\x18\x01 \x01(\x0b\x32\x1c.bitswap.pb.Message.Wantlist\x12\x0e\n\x06\x62locks\x18\x02 \x03(\x0c\x12*\n\x07payload\x18\x03 \x03(\x0b\x32\x19.bitswap.pb.Message.Block\x12\x39\n\x0e\x62lockPresences\x18\x04 \x03(\x0b\x32!.bitswap.pb.Message.BlockPresence\x12\x14\n\x0cpendingBytes\x18\x05 \x01(\x05\x1a\xf8\x01\n\x08Wantlist\x12\x33\n\x07\x65ntries\x18\x01 \x03(\x0b\x32\".bitswap.pb.Message.Wantlist.Entry\x12\x0c\n\x04\x66ull\x18\x02 \x01(\x08\x1a\x87\x01\n\x05\x45ntry\x12\r\n\x05\x62lock\x18\x01 \x01(\x0c\x12\x10\n\x08priority\x18\x02 \x01(\x05\x12\x0e\n\x06\x63\x61ncel\x18\x03 \x01(\x08\x12\x37\n\x08wantType\x18\x04 \x01(\x0e\x32%.bitswap.pb.Message.Wantlist.WantType\x12\x14\n\x0csendDontHave\x18\x05 \x01(\x08\"\x1f\n\x08WantType\x12\t\n\x05\x42lock\x10\x00\x12\x08\n\x04Have\x10\x01\x1a%\n\x05\x42lock\x12\x0e\n\x06prefix\x18\x01 \x01(\x0c\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x1aQ\n\rBlockPresence\x12\x0b\n\x03\x63id\x18\x01 \x01(\x0c\x12\x33\n\x04type\x18\x02 \x01(\x0e\x32%.bitswap.pb.Message.BlockPresenceType\"+\n\x11\x42lockPresenceType\x12\x08\n\x04Have\x10\x00\x12\x0c\n\x08\x44ontHave\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nanolib.bitswap.pb.bitswap_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_MESSAGE']._serialized_start=48
  _globals['_MESSAGE']._serialized_end=664
  _globals['_MESSAGE_WANTLIST']._serialized_start=249
  _globals['_MESSAGE_WANTLIST']._serialized_end=497
  _globals['_MESSAGE_WANTLIST_ENTRY']._serialized_start=329
  _globals['_MESSAGE_WANTLIST_ENTRY']._serialized_end=464
  _globals['_MESSAGE_WANTLIST_WANTTYPE']._serialized_start=466
  _globals['_MESSAGE_WANTLIST_WANTTYPE']._serialized_end=497
  _globals['_MESSAGE_BLOCK']._serialized_start=499
  _globals['_MESSAGE_BLOCK']._serialized_end=536
  _globals['_MESSAGE_BLOCKPRESENCE']._serialized_start=538
  _globals['_MESSAGE_BLOCKPRESENCE']._serialized_end=619
  _globals['_MESSAGE_BLOCKPRESENCETYPE']._serialized_start=621
  _globals['_MESSAGE_BLOCKPRESENCETYPE']._serialized_end=664
# @@protoc_insertion_point(module_scope)
